# encoding: utf-8
require 'smartdown/model/answer/money'

describe Smartdown::Model::Answer::Money do

  let(:money_input) { '1523.42' }
  subject(:instance) { described_class.new(money_input) }

  specify { expect(instance.value).to eql(1523.42) }

  describe 'instantiate with integer' do
    let(:money_input) { 1523 }

    specify { expect(instance.value).to eql(1523.0) }
  end

  describe 'instantiate with float' do
    let(:money_input) { 1523.12 }

    specify { expect(instance.value).to eql(1523.12) }
  end

  describe 'instantiate with £' do
    let(:money_input) { '£1523.45' }

    specify { expect(instance.value).to eql(1523.45) }
  end

  describe 'instantiate with float with a trailing dot' do
    let(:money_input) { '1523.' }

    specify { expect(instance.value).to eql(1523.0) }
  end

  describe 'to_s' do
    it "returns value without comma delimiter" do
      expect(instance.to_s).to eql("1523.42")
    end
  end

  describe "#humanize" do
    it "specifies money in the correct format" do
      expect(instance.humanize).to eql("£1,523.42")
    end

    context "rounding up" do
      let(:money_input) { '1523.427' }
      it "rounds up amounts of money correctly" do
        expect(instance.humanize).to eql("£1,523.43")
      end
    end
    context "rounding down" do
      let(:money_input) { '1523.421' }
      it "rounds down amounts of money correctly" do
        expect(instance.humanize).to eql("£1,523.42")
      end
    end
    context "rounds down in the .005 case" do
      let(:money_input) { '1523.425' }
      it "rounds down amounts of money correctly" do
        expect(instance.humanize).to eql("£1,523.42")
      end
    end

    context "no pence" do
      let(:money_input) { '1523.00' }
      it "rounds down amounts of money correctly" do
        expect(instance.humanize).to eql("£1,523")
      end
    end

    context "for value with a trailing dot" do
      let(:money_input) { '1523.' }
      it "ignores the trailing dot" do
        expect(instance.humanize).to eql("£1,523")
      end
    end
  end

  describe "parsing" do
    context "no digits" do
      let(:money_input) {"Loads'a'money"}

      it "raises an error" do
        expect(instance.error).to eql("Invalid format")
      end
    end

    context "empty input" do
      let(:money_input) { nil }

      it "raises an error" do
        expect(instance.error).to eql("Please answer this question")
      end
    end
  end

  describe "comparisons" do
    let(:money_input) { '62,400' }

    context "comparing against Float" do
      specify { expect(instance == 62400.0).to eql true }

      specify { expect(instance < 62400.1).to eql true }
      specify { expect(instance < 62400.0).to eql false }
      specify { expect(instance < 62399.9).to eql false }

      specify { expect(instance > 62400.1).to eql false }
      specify { expect(instance > 62400.0).to eql false }
      specify { expect(instance > 62399.9).to eql true }

      specify { expect(instance <= 62400.1).to eql true }
      specify { expect(instance <= 62400.0).to eql true }
      specify { expect(instance <= 62399.9).to eql false }

      specify { expect(instance >= 62400.1).to eql false }
      specify { expect(instance >= 62400.0).to eql true }
      specify { expect(instance >= 62399.9).to eql true }
    end

    context "comparing against Integer" do
      specify { expect(instance == 62400).to eql true }

      specify { expect(instance < 62401).to eql true }
      specify { expect(instance < 62400).to eql false }
      specify { expect(instance < 62399).to eql false }

      specify { expect(instance > 62401).to eql false }
      specify { expect(instance > 62400).to eql false }
      specify { expect(instance > 62399).to eql true }

      specify { expect(instance <= 62401).to eql true }
      specify { expect(instance <= 62400).to eql true }
      specify { expect(instance <= 62399).to eql false }

      specify { expect(instance >= 62401).to eql false }
      specify { expect(instance >= 62400).to eql true }
      specify { expect(instance >= 62399).to eql true }
    end

    context "comparing against strings" do
      specify { expect(instance == '62400').to eql true }
      specify { expect(instance < '62400.1').to eql true }
      specify { expect(instance > '62400').to eql false }
      specify { expect(instance <= '62400.1').to eql true }
      specify { expect(instance >= '62400').to eql true }
    end

    context "comparing against Answer::Money" do
      specify { expect(instance == described_class.new(money_input)).to eql true }

      specify { expect(instance < described_class.new('62,400.1')).to eql true }
      specify { expect(instance < described_class.new('62,400')).to eql false }
      specify { expect(instance < described_class.new('62,399.99')).to eql false }

      specify { expect(instance > described_class.new('62,400.1')).to eql false }
      specify { expect(instance > described_class.new('62,400')).to eql false }
      specify { expect(instance > described_class.new('62,399.99')).to eql true }

      specify { expect(instance <= described_class.new('62,400.1')).to eql true }
      specify { expect(instance <= described_class.new('62,400')).to eql true }
      specify { expect(instance <= described_class.new('62,399.99')).to eql false }

      specify { expect(instance >= described_class.new('62,400.1')).to eql false }
      specify { expect(instance >= described_class.new('62,400')).to eql true }
      specify { expect(instance >= described_class.new('62,399.99')).to eql true }
    end
  end
end
